@@ -10,6 +10,7 @@ require_once 'confusa_constants.php';
 require_once 'CGE_ComodoCredentialException.php';
 require_once 'CurlWrapper.php';
 require_once 'CS.php';
+require_once 'NRENAccount.php';
 
 /**
  * CA_Comodo. Comodo signing extension for CA.
@@ -19,24 +20,20 @@ require_once 'CS.php';
  *
  * PHP version 5
  * @author: Thomas Zangerl <tzangerl@pdc.kth.se>
+ * @author: Henrik Austad <henrik@austad.us>
  */
 class CA_Comodo extends CA
 {
     /* order number for communication with the remote API */
     private $order_number;
-
-    /* login-name and password for the remote-signing CA */
-    private $login_name;
-    private $login_pw;
-    /* alliance-partner name for the remote-signing CA */
-    private $ap_name;
+	private $account;
 
 
     function __construct($pers, $validityPeriod)
     {
         parent::__construct($pers, $validityPeriod);
-        $this->getAccountInformation();
 
+		$this->account = NRENAccount::get($pers);
 		if (Config::get_config('capi_test') == true) {
 			$this->dcs[] = ConfusaConstants::$CAPI_TEST_DC_PREFIX;
 		}
@@ -48,72 +45,6 @@ class CA_Comodo extends CA
 		}
     }
 
-    /**
-     * Get username and password for the remote-CA account of the
-     * (NREN of) the managed person.
-     */
-    private function getAccountInformation() {
-
-		$nren = $this->person->getNREN();
-
-		/* can only get the account if we have NREN information */
-		if (empty($nren)) {
-			return;
-		}
-
-        $login_cred_query = "SELECT a.account_login_name, a.account_password, a.account_ivector, a.ap_name " .
-              "FROM nren_account_map_view a WHERE a.nren=?";
-
-        $nren = $this->person->getNREN();
-	if (Config::get_config('debug')) {
-		Logger::log_event(LOG_INFO, __CLASS__ . "::" . __FUNCTION__ .
-				  " Getting the remote-CA login " .
-				  "credentials for NREN " .
-				  $nren
-			);
-	}
-	try {
-		$errorCode = PW::create(8);
-		$errorMsg = "[$errorCode] " . __FILE__ . ":" . __LINE__ . " ";
-
-		$res = MDB2Wrapper::execute($login_cred_query, array('text'),
-					    array($nren));
-	} catch (DBStatementException $dbse) {
-		Logger::log_event(LOG_ALERT, "$errorMsg missing columns in account_map/nren_account_map_view");
-		$errorMsg .= "<br />The table does not have all required columns. Please contact operational support.";
-		throw new CGE_ComodoCredentialException($errorMsg);
-	} catch (DBQueryException $dbqe) {
-		if (is_null($nren) || $nren =="") {
-			Logger::log_event(LOG_NOTICE, "$errorMsg - Look for subscriber-map problems.");
-			$errorMsg .= "<br />NREN-name not properly set, cannot extract account-credentials.";
-			throw new CGE_ComodoCredentialException($errorMsg);
-		} else {
-			$errorMsg .= " unknown query-inconsistency";
-			Logger::log_event(LOG_NOTICE, $errorMsg);
-			throw new CGE_ComodoCredentialException($errorMsg);
-		}
-	}
-        if (count($res) != 1) {
-            Logger::log_event(LOG_NOTICE, "Could not extract the suitable remote CA credentials for NREN $nren!");
-            throw new CGE_ComodoCredentialException("Could not extract the suitable " .
-                           "remote CA credentials for NREN " . $this->person->getNREN() . "!\n");
-        }
-
-        $this->login_name = $res[0]['account_login_name'];
-        $this->ap_name = $res[0]['ap_name'];
-
-		if (!isset($this->login_name) || !isset($this->ap_name)) {
-			return;
-		}
-
-        $encrypted_pw = base64_decode($res[0]['account_password']);
-        $ivector = base64_decode($res[0]['account_ivector']);
-        $encryption_key = Config::get_config('capi_enc_pw');
-        $this->login_pw = trim(base64_decode(mcrypt_decrypt(
-                                MCRYPT_RIJNDAEL_256, $encryption_key,
-                                $encrypted_pw, MCRYPT_MODE_CFB,
-                                $ivector)));
-    }
 	/**
 	 * Set an expiry date on the cache based on the time of the latest
 	 * certificate order. The idea is that the more recently a certificate has
@@ -228,9 +159,8 @@ class CA_Comodo extends CA
 		}
 
 		$authKey = $csr->getAuthToken();
-
-		Logger::logEvent(LOG_DEBUG, __CLASS__, "signKey()",
-		                 "Signing CSR with authKey $authKey",
+		Logger::logEvent(LOG_INFO, __CLASS__, "signKey()",
+		                 "Preparing to sign CSR ($authKey) " .$this->owner_string,
 		                 __LINE__);
 
 		/* FIXME: better solution */
@@ -255,7 +185,7 @@ class CA_Comodo extends CA
 		                         $_SERVER['REMOTE_ADDR'],
 		                         $this->person,
 		                         $this->getFullDN());
-		Logger::log_event(LOG_INFO, "Signed CSR for user with authKey $authKey");
+		Logger::log_event(LOG_INFO, "Successfully signed new certificate. ". $this->owner_string);
 		return $this->order_number;
 	} /* end signKey() */
 
@@ -550,6 +480,25 @@ class CA_Comodo extends CA
 		}
 		return $res;
 	} /* end getCertListForEPPN */
+	/**
+	 * verifyCredentials() validate username/password to Comodo
+	 *
+	 * @param String $username
+	 * @param String $password
+	 * @return Boolean true if username/password was ok
+	 */
+	function verifyCredentials($username, $password)
+	{
+		require_once "pw.php";
+		require_once "CurlWrapper.php";
+		$pf = $this->bs_pf();
+		$pf["commonName"]    = "".PW::create(32);
+		$data = CurlWrapper::curlContact(ConfusaConstants::$CAPI_LISTING_ENDPOINT, "post", $pf);
+		parse_str($data, $params);
+		if (array_key_exists('errorCode', $params) && $params['errorCode'] === "0")
+			return true;
+		return false;
+	}
 
     /**
      * Return true if processing of the certificate is finished and false
@@ -563,10 +512,10 @@ class CA_Comodo extends CA
         $key = $this->transformToOrderNumber($key);
 
         $polling_endpoint = ConfusaConstants::$CAPI_COLLECT_ENDPOINT .
-                        "?loginName=" . $this->login_name .
-                        "&loginPassword=" . $this->login_pw .
-                        "&orderNumber=" . $key .
-                        "&queryType=0";
+			"?loginName="     . $this->account->getLoginName() .
+			"&loginPassword=" . $this->account->getPassword(true) .
+			"&orderNumber="   . $key .
+			"&queryType=0";
 
 		$data = CurlWrapper::curlContact($polling_endpoint);
 
@@ -592,17 +541,16 @@ class CA_Comodo extends CA
     {
         $key = $this->transformToOrderNumber($key);
 
-        Logger::log_event(LOG_NOTICE, "Trying to retrieve certificate with order number " .
-                                      $key .
-                                      " from the Comodo collect API. Sending to user with ip " .
-                                      $_SERVER['REMOTE_ADDR']);
+        Logger::log_event(LOG_DEBUG, "Trying to retrieve certificate with order number " .
+						  $key .
+						  " from the Comodo collect API. " . $this->owner_string);
 
         $collect_endpoint = ConfusaConstants::$CAPI_COLLECT_ENDPOINT .
-                            "?loginName=" . $this->login_name .
-                            "&loginPassword=" . $this->login_pw .
-                            "&orderNumber=" . $key .
-                            "&queryType=2" .
-                            "&responseMimeType=application/x-x509-user-cert";
+			"?loginName="     . $this->account->getLoginName() .
+			"&loginPassword=" . $this->account->getPassword(true) .
+			"&orderNumber=" . $key .
+			"&queryType=2" .
+			"&responseMimeType=application/x-x509-user-cert";
 
 		$data = CurlWrapper::curlContact($collect_endpoint);
 
@@ -653,24 +601,20 @@ class CA_Comodo extends CA
         $return_res = NULL;
 
         Logger::log_event(LOG_NOTICE, "Revoking certificate with order number " .
-                                      $key .
-                                      " using Comodo's auto-revoke-API. Sending to user with ip " .
-                                      $_SERVER['REMOTE_ADDR']);
+						  $key ." using Comodo's auto-revoke-API. " . $this->owner_string);
 
         $revoke_endpoint = ConfusaConstants::$CAPI_REVOKE_ENDPOINT;
-        $postfields_revoke = array();
-        $postfields_revoke["loginName"] = $this->login_name;
-        $postfields_revoke["loginPassword"] = $this->login_pw;
+        $postfields_revoke = $this->bs_pf();
         $postfields_revoke["revocationReason"] = $reason;
-        $postfields_revoke["orderNumber"] = $key;
-        $postfields_revoke["includeInCRL"] = 'Y';
+        $postfields_revoke["orderNumber"]      = $key;
+        $postfields_revoke["includeInCRL"]     = 'Y';
 
         /* will not revoke test certificates? */
         if (Config::get_config('capi_test')) {
-		Logger::log_event(LOG_DEBUG, "CA_C: in test-mode");
-		$postfields_revoke["test"] = 'Y';
+			Logger::log_event(LOG_DEBUG, "CA_C: in test-mode");
+			$postfields_revoke["test"] = 'Y';
         }
-	$data = CurlWrapper::curlContact($revoke_endpoint, "post", $postfields_revoke);
+		$data = CurlWrapper::curlContact($revoke_endpoint, "post", $postfields_revoke);
 
         /* try to catch all kinds of errors that can happen when connecting */
         if ($data === FALSE) {
@@ -694,15 +638,13 @@ class CA_Comodo extends CA
 				CS::deleteSessionKey('rawCertList');
 				Logger::log_event(LOG_NOTICE, "Revoked certificate with " .
 								  "order number $key using Comodo's AutoRevoke " .
-								  "API. User contacted us from " .
-								  $_SERVER['REMOTE_ADDR']);
+								  "API. " . $this->owner_string);
 				return true;
 				break;
 			default:
 				$msg = $this->capiErrorMessage($error_parts[0], $error_parts[1]);
-				Logger::log_event(LOG_ERROR, "Revocation of certificate with " .
-				 "order_number $key failed! User contacted us from " .
-				 $_SERVER['REMOTE_ADDR']);
+				Logger::log_event(LOG_ERR, "Revocation of certificate with " .
+								  "order_number $key failed! ". $this->owner_string);
 				throw new CGE_ComodoAPIException("Received error message $data. $msg");
 				break;
 			}
@@ -720,10 +662,8 @@ class CA_Comodo extends CA
 		$key = $this->transformToOrderNumber($key);
 
 		$list_endpoint = ConfusaConstants::$CAPI_LISTING_ENDPOINT;
-		$postfields_list = array();
-        $postfields_list["loginName"]		= $this->login_name;
-        $postfields_list["loginPassword"]	= $this->login_pw;
-        $postfields_list["orderNumber"] = $key;
+		$postfields_list = $this->bs_pf();
+        $postfields_list["orderNumber"]		= $key;
         $data = CurlWrapper::curlContact($list_endpoint, "post", $postfields_list);
 		$params = array();
 		parse_str($data, $params);
@@ -779,31 +719,30 @@ class CA_Comodo extends CA
         switch ($browser) {
         case "msie_post_vista":
             $collect_endpoint = ConfusaConstants::$CAPI_COLLECT_ENDPOINT .
-                                   "?loginName=" . $this->login_name .
-                                "&loginPassword=" . $this->login_pw .
-                                "&orderNumber=" . $key .
-                                "&queryType=1" .
-                                "&responseType=2" . /* PKCS#7 */
-                                "&responseEncoding=2" . /* encode in Javascript */
-                                "&responseMimeType=text/javascript" .
-                                /* call that function after the JS variable-declarations */
-                                "&callbackFunctionName=installIEVistaCertificate";
-
+				"?loginName=" . $this->account->getLoginName() .
+				"&loginPassword=" . $this->account->getPassword(true) .
+				"&orderNumber=" . $key .
+				"&queryType=1" .
+				"&responseType=2" . /* PKCS#7 */
+				"&responseEncoding=2" . /* encode in Javascript */
+				"&responseMimeType=text/javascript" .
+				/* call that function after the JS variable-declarations */
+				"&callbackFunctionName=installIEVistaCertificate";
 			$data = CurlWrapper::curlContact($collect_endpoint);
             return "<script type=\"text/javascript\">$data</script>";
             break;
 
         case "msie_pre_vista":
             $collect_endpoint = ConfusaConstants::$CAPI_COLLECT_ENDPOINT .
-                                   "?loginName=" . $this->login_name .
-                                "&loginPassword=" . $this->login_pw .
-                                "&orderNumber=" . $key .
-                                "&queryType=1" .
-                                "&responseType=2" . /* PKCS#7 */
-                                "&responseEncoding=2" . /* encode in Javascript */
-                                "&responseMimeType=text/javascript" .
-                                /* call that function after the JS variable-declarations */
-                                "&callbackFunctionName=installIEXPCertificate";
+				"?loginName="     . $this->account->getLoginName() .
+				"&loginPassword=" . $this->account->getPassword(true) .
+				"&orderNumber="   . $key .
+				"&queryType=1" .
+				"&responseType=2" . /* PKCS#7 */
+				"&responseEncoding=2" . /* encode in Javascript */
+				"&responseMimeType=text/javascript" .
+				/* call that function after the JS variable-declarations */
+				"&callbackFunctionName=installIEXPCertificate";
 
 			$data = CurlWrapper::curlContact($collect_endpoint);
             return "<script type=\"text/javascript\">$data</script>";
@@ -811,12 +750,12 @@ class CA_Comodo extends CA
 
         case "chrome":
 			 $collect_endpoint = ConfusaConstants::$CAPI_COLLECT_ENDPOINT .
-                                   "?loginName=" . $this->login_name .
-                                    "&loginPassword=" . $this->login_pw .
-                                    "&orderNumber=" . $key .
-                                    "&queryType=2" .
-                                    "&responseType=3" . /* PKCS#7 */
-                                    "&responseEncoding=0"; /* encode base-64 */
+				 "?loginName="     . $this->account->getLoginName() .
+				 "&loginPassword=" . $this->account->getPassword(true) .
+				 "&orderNumber="   . $key .
+				 "&queryType=2" .
+				 "&responseType=3" . /* PKCS#7 */
+				 "&responseEncoding=0"; /* encode base-64 */
 
             $data = CurlWrapper::curlContact($collect_endpoint);
             $cert = new Certificate(trim(substr($data, 2)));
@@ -828,12 +767,12 @@ class CA_Comodo extends CA
         case "safari":
         case "opera":
             $collect_endpoint = ConfusaConstants::$CAPI_COLLECT_ENDPOINT .
-                                   "?loginName=" . $this->login_name .
-                                    "&loginPassword=" . $this->login_pw .
-                                    "&orderNumber=" . $key .
-                                    "&queryType=2" .
-                                    "&responseType=3" . /* PKCS#7 */
-                                    "&responseEncoding=0"; /* encode base-64 */
+			"?loginName="     . $this->account->getLoginName() .
+			"&loginPassword=" . $this->account->getPassword(true) .
+			"&orderNumber="   . $key .
+			"&queryType=2" .
+			"&responseType=3" . /* PKCS#7 */
+			"&responseEncoding=0"; /* encode base-64 */
 
 			$data = CurlWrapper::curlContact($collect_endpoint);
             return trim(substr($data,2));
@@ -850,8 +789,7 @@ class CA_Comodo extends CA
 		Logger::log_event(LOG_DEBUG, "Trying to get the the list of the certificates " .
 		                             "for user $eppn");
 		$list_endpoint = ConfusaConstants::$CAPI_LISTING_ENDPOINT;
-		$postfields_list["loginName"]		= $this->login_name;
-		$postfields_list["loginPassword"]	= $this->login_pw;
+		$postfields_list = $this->bs_pf();
 
 		if (Config::get_config('cert_product') == PRD_ESCIENCE) {
 			/* be sure to match the EPPN only and not also its suffix. For
@@ -902,9 +840,8 @@ class CA_Comodo extends CA
                                     "for person $common_name");
 
         $list_endpoint = ConfusaConstants::$CAPI_LISTING_ENDPOINT;
-        $postfields_list["loginName"]		= $this->login_name;
-        $postfields_list["loginPassword"]	= $this->login_pw;
-        $postfields_list["commonName"]		= $common_name;
+		$postfields_list = $this->bs_pf();
+		$postfields_list["commonName"]		= $common_name;
         $postfields_list["notBefore"]		= time() - $days*24*3600;
 
         $data = CurlWrapper::curlContact($list_endpoint, "post", $postfields_list);
@@ -942,8 +879,7 @@ class CA_Comodo extends CA
 							"for organization $organization");
 
 		$listEndpoint = ConfusaConstants::$CAPI_LISTING_ENDPOINT;
-		$postfieldsList["loginName"]		= $this->login_name;
-		$postfieldsList["loginPassword"]	= $this->login_pw;
+		$postfieldsList = $this->bs_pf();
 		$postfieldsList["organizationName"]	= $organization;
 		$postfieldsList["notBefore"] 		= time() - $days*24*3600;
 
@@ -985,6 +921,7 @@ class CA_Comodo extends CA
 		} else if (Config::get_config('cert_product') == PRD_ESCIENCE) {
 			$ca_cert_id = ConfusaConstants::$CAPI_ESCIENCE_ID;
 		} else {
+			Logger::log_event(LOG_INFO, "Confusa is not configured properly, please fix product-mode");
 			throw new KeySignException("Confusa's configured product-mode is " .
 			                           "illegal! Must be one of: PRD_ESCIENCE, " .
 			                           "PRD_PERSONAL. Please contact an IT " .
@@ -993,15 +930,15 @@ class CA_Comodo extends CA
 		$orgName = $this->person->getSubscriber()->getOrgName();
 
         $postfields_sign_req=array();
-	$pf_counter = 1;
+		$pf_counter = 1;
 
         /* set all the required post parameters for upload */
-        $postfields_sign_req["ap"] = $this->ap_name;
-        $postfields_sign_req[$csr_format] = $csr;
-        $postfields_sign_req["days"] = $this->validityDays;
-        $postfields_sign_req["successURL"] = "none";
-        $postfields_sign_req["errorURL"] = "none";
-        $postfields_sign_req["caCertificateId"] = $ca_cert_id;
+		$postfields_sign_req["ap"] = $this->account->getAPName();
+	$postfields_sign_req[$csr_format] = $csr;
+	$postfields_sign_req["days"] = $this->validityDays;
+	$postfields_sign_req["successURL"] = "none";
+	$postfields_sign_req["errorURL"] = "none";
+	$postfields_sign_req["caCertificateId"] = $ca_cert_id;
 
 	$cert_email_option = $this->person->getNREN()->getEnableEmail();
 	$rce = $this->person->getRegCertEmails();
@@ -1053,12 +990,12 @@ class CA_Comodo extends CA
 		/* manually compose the subject. Necessary, because we want to have
          * Terena domainComponents */
         $postfields_sign_req["subject_commonName_$pf_counter"] =
-		stripslashes($this->person->getX509ValidCN());
+			stripslashes($this->person->getX509ValidCN());
 		$pf_counter++;
 
         $postfields_sign_req["subject_organizationName_".$pf_counter++] = $orgName;
         $postfields_sign_req["subject_countryName_".$pf_counter++] =
-		$this->person->getNREN()->getCountry();
+			$this->person->getNREN()->getCountry();
 
 	foreach(array_reverse($this->dcs) as $dc) {
 		if ($dc == ConfusaConstants::$CAPI_TEST_DC_PREFIX)
@@ -1068,7 +1005,7 @@ class CA_Comodo extends CA
 	if (Config::get_config('capi_test') == true) {
 		$postfields_sign_req["subject_domainComponent_".$pf_counter++] = ConfusaConstants::$CAPI_TEST_DC_PREFIX;
 	}
-
+	Logger::log_event(LOG_DEBUG, "Pushing new CSR to Comodo for " . $this->person->getX509ValidCN());
 	$data = CurlWrapper::curlContact($sign_endpoint, "post", $postfields_sign_req);
 
         $params=array();
@@ -1076,9 +1013,11 @@ class CA_Comodo extends CA
         /*
          * If something has failed, an errorCode parameter will be set in
          * the return message
+         *
+         * FIXME: add l10n to these messages
          */
         if (isset($params['errorCode'])) {
-		$msg = "Received an error when uploading the CSR to the remote CA: ";
+			$msg = "Received an error when uploading the CSR to the remote CA: ";
 		if (isset($params['errorMessage'])) {
 			$msg .= " " . $params['errorMessage'];
 		}
@@ -1086,6 +1025,9 @@ class CA_Comodo extends CA
 			$msg .= " " . $params['errorItem'];
 		}
 		$msg .= $this->capiErrorMessage($params['errorCode'], $params['errorMessage']);
+		Logger::log_event(LOG_WARN, "Error when uploading CSR, got " .
+						  $params['errorMessage'] . "(" .
+						  $params['errorItem'] . ")");
 		throw new CGE_ComodoAPIException($msg);
         }
 
@@ -1094,18 +1036,14 @@ class CA_Comodo extends CA
             if (!isset($params['orderNumber'])) {
                 $msg = "Response looks malformed. Maybe there is a configuration " .
                        "error in Confusa's Comodo-CA configuration!";
+				Logger::log_event(LOG_WARN, "Malformed response from Comodo, no error or orderNumber received.");
                 throw new CGE_ComodoAPIException($msg);
             }
 
             $this->order_number = $params['orderNumber'];
 
-            Logger::log_event(LOG_INFO, "Uploaded CSR to remote CA. Received " .
-                                        "order number " .
-                                        $this->order_number .
-                                        " for user " .
-                                        stripslashes($this->person->getX509ValidCN()) .
-                                        " Person contacted us from " .
-                                        $_SERVER['REMOTE_ADDR']);
+			Logger::log_event(LOG_INFO, "Successfully uploaded CSR to remote CA, got ".
+							  $this->order_number . " " . $this->owner_string);
 
           $sql_command= "INSERT INTO order_store(auth_key, owner, " .
                         "order_number, order_date, authorized)" .
@@ -1209,12 +1147,10 @@ class CA_Comodo extends CA
     private function capiAuthorizeCSR()
     {
         $authorize_endpoint = ConfusaConstants::$CAPI_AUTH_ENDPOINT;
-
-        $postfields_auth = array();
-        $postfields_auth["loginName"] = $this->login_name;
-        $postfields_auth["loginPassword"] = $this->login_pw;
+		$postfields_auth =$this->bs_pf();
         $postfields_auth["orderNumber"] = $this->order_number;
 		$data = CurlWrapper::curlContact($authorize_endpoint, "post", $postfields_auth);
+		Logger::log_event(LOG_DEBUG, "Authorizing CSR " . $this->order_number . " for signing.");
 
         /* the only formal restriction we have is if the API returns 0 for the query */
         if (substr($data,0,1) == "0") {
@@ -1222,20 +1158,31 @@ class CA_Comodo extends CA
           MDB2Wrapper::update("UPDATE order_store SET authorized='authorized' WHERE order_number=?",
                               array('text'),
                               array($this->order_number));
-          Logger::log_event(LOG_NOTICE, "Authorized remote certificate for person ".
-                                        stripslashes($this->person->getX509ValidCN()).
-                                        " with order number " .
-                                        $this->order_number .
-                                        " Person contacted us from ".
-                                        $_SERVER['REMOTE_ADDR']);
+          Logger::log_event(LOG_NOTICE, "Authorized certificate with order number " .
+							$this->order_number . ". " . $this->owner_string);
         } else {
+			Logger::log_event(LOG_WARNING,
+							  "Error authorizing CSR ".$this->order_number. " ".
+							  "Server said " . $error_parts[0] ." (".$error_parts[1].")");
             $msg = "Received an error when authorizing the CSR with orderNumber " .
                    $this->order_number . $data . "\n";
 				   $error_parts = explode("\n", $data, 2);
 			$msg .= $this->capiErrorMessage($error_parts[0], $error_parts[1]);
             throw new CGE_ComodoAPIException($msg);
         }
-
     } /* end capiAuthorizeCSR */
+
+	/**
+	 * bs_pf() Bootstrap Postfields
+	 */
+	private function bs_pf()
+	{
+        $pf                  = array();
+        $pf["loginName"]     = $this->account->getLoginName();
+		/* FIXME: use true as argument, password *should* be urlencoded! (But
+		 * comodo will bomb */
+        $pf["loginPassword"] = $this->account->getPassword(false);
+		return $pf;
+	}
 } /* end class CA_Comodo */
 ?>
